/*
 * Copyright (C) 2005 Texas Instruments.
 *
 * (C) Copyright 2004
 * Jian Zhang, Texas Instruments, jzhang@ti.com.
 *
 * (C) Copyright 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/mem.h>
#include <version_autogenerated.h>
#include <timestamp_autogenerated.h>

extern int misc_init_r (void);
extern u32 get_mem_type(void);

#ifdef CFG_PRINTF
int print_info(void)
{
#ifdef CFG_BOARD_NAME
	printf("\n\nTexas Instruments " X_LOADER_VERSION " for %s ("
		X_LOADER_DATE " " X_LOADER_TIME ")\n", CFG_BOARD_NAME);
#else
	printf("\n\nTexas Instruments " X_LOADER_VERSION " ("
		X_LOADER_DATE " " X_LOADER_TIME ")\n");
#endif
	return 0;
}
#endif

typedef int (init_fnc_t) (void);

init_fnc_t *init_sequence[] = {
	cpu_init,		/* basic cpu dependent setup */
	board_init,		/* basic board dependent setup */
#ifdef CFG_PRINTF
 	serial_init,		/* serial communications setup */
	print_info,
#endif
  	nand_init,		/* board specific nand init */
  	NULL,
};

void start_armboot (void)
{
  	init_fnc_t *boot_fnc_ptr = (init_fnc_t *)CFG_LOADADDR;
  	init_fnc_t **init_fnc_ptr;
	unsigned int size = 0;
 	int i;
	uchar *buf;

   	for (init_fnc_ptr = init_sequence; *init_fnc_ptr; ++init_fnc_ptr) {
		if ((*init_fnc_ptr)() != 0) {
			hang ();
		}
	}

	misc_init_r();
	buf =  (uchar*) CFG_LOADADDR;

#ifdef CFG_HANG_BDI
	printf("%s: Hang here waiting for BDI to connect\n", __FUNCTION__);
	hang();
#endif
	

#ifdef CONFIG_DM3730LOGIC
	/* If we booted from MMC, then look on MMC for the image.  Determine
	 * if booted from SDMMC by looking at VECT_HI from bootrom left in SRAM
	 * to see what device processor booted from */
	if (omap_boot_device() == boot_device_sdmmc) {
		size = mmc_boot(buf);
		if (size > 0) {
			buf += size;
			goto boot_it;
		}
	}
#endif

	if ((get_mem_type() == MMC_ONENAND) || (get_mem_type() == MMC_NAND)){
		printf("%s:%d\n", __FUNCTION__, __LINE__);
		buf += mmc_boot(buf);
	}

#ifdef ONENAND_START_BLOCK
	if (get_mem_type() == GPMC_ONENAND){
		for (i = ONENAND_START_BLOCK; i < ONENAND_END_BLOCK; i++){
			if (!onenand_read_block(buf, i))
				buf += ONENAND_BLOCK_SIZE;
		}
	}
#endif

	if (get_mem_type() == GPMC_NAND){
		printf("%s:%d\n", __FUNCTION__, __LINE__);
		for (i = NAND_UBOOT_START; i < NAND_UBOOT_END; i+= NAND_BLOCK_SIZE){
			if (!nand_read_block(buf, i))
				buf += NAND_BLOCK_SIZE; /* advance buf ptr */
		}
	}


boot_it:

	if (buf == (uchar *)CFG_LOADADDR)
		hang();

	/* go run U-Boot and never return */
  	printf("Starting OS Bootloader at %p...\n", (uchar*)CFG_LOADADDR);
 	((init_fnc_t *)CFG_LOADADDR)();

	/* should never come here */
}

void hang (void)
{
	/* call board specific hang function */
	board_hang();

	/* if board_hang() returns, hange here */
	printf("X-Loader hangs\n");
	for (;;);
}
